# -*- coding: utf-8 -*-
#
#  OX for UCS
#  listener module: handling fetchmail accounts
#
# Copyright (C) 2004-2014 Univention GmbH
#
# http://www.univention.de/
#
# and
#
# iKu Systems & Services GmbH & Co. KG
# http://www.iku-systems.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

__package__ = '' 	# workaround for PEP 366
import listener
import univention.uldap
import univention.debug

import os

name='ox-fetchmail'
description='Update Open-Xchange fetchmailrc'
filter='(&(|(objectClass=oxFetchmailMultiDrop)(objectClass=oxFetchmailSingleDrop))(cn=*))'
attributes=[]

ldapbase=listener.baseConfig['ldap/base']

fetchmailrc='/etc/fetchmailrc'


def initialize():
	pass


def handler(dn, new, old):
	fetchmailrcline1=''
	fetchmailrcline2=''
	fetchmailrcline3=''
	fetchmailrcline4=''

	listener.setuid(0)
	try:
		ldapCon = univention.uldap.getMachineConnection(ldap_master = False)

		deletedcn=''
		if old and old.has_key('cn') and not new:
			deletedcn=old['cn']

		f=open(fetchmailrc, 'w')

		f.write('# Do not edit this file!\n# This file will by overwritten by Univention Directory Manager\nset syslog\nset daemon 300\n')
		result_id = ldapCon.search(base="cn=users,%s" % ldapbase, filter='uid=oxadmin', attr=['mailPrimaryAddress'])
		if result_id:
			# adminmail=result_id[1]['mailPrimaryAddress']
			adminmail=result_id[0][1]['mailPrimaryAddress'][0]
			f.write('set postmaster "%s"\n\n' % adminmail )
		result_id = ldapCon.search(base="cn=mail,%s" % ldapbase, filter=filter, attr=['*'])
		if result_id:

			for result in result_id:
				try:
					univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'ox-fetchmail: result: %s' % result[1])
					if result[1].has_key('cn') and result[1]['cn']:
						thiscn=result[1]['cn']
						univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'ox-fetchmail: thiscn: %s' % thiscn)
					if result[1] and result[1].has_key('cn') and result[1]['cn']:
						cn=result[1]['cn'][0]
						univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'ox-fetchmail: cn: %s' % cn)
						fetchmailrcline1=('# identifier %s' % cn)

					if result[1] and result[1].has_key('remoteServer') and result[1]['remoteServer']:
						remoteServer=result[1]['remoteServer'][0]
						univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'ox-fetchmail: remoteServer: %s' % remoteServer)
						fetchmailrcline2=('poll \'%s\'' % remoteServer)

					if result[1] and result[1].has_key('protocol') and result[1]['protocol']:
						protocol=result[1]['protocol'][0]
						univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'ox-fetchmail: protocol: %s' % protocol)
						fetchmailrcline2=('%s with proto %s' % (fetchmailrcline2,protocol))

					if result[1] and result[1].has_key('envelopeHeader') and result[1]['envelopeHeader']:
						envelopeHeader=result[1]['envelopeHeader'][0]
						univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'ox-fetchmail: envelopeHeader: %s' % envelopeHeader)
						fetchmailrcline2=('%s envelope %s' % (fetchmailrcline2,envelopeHeader))

					if result[1] and result[1].has_key('qmailPrefix') and result[1]['qmailPrefix']:
						qmailPrefix=result[1]['qmailPrefix'][0]
						univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'ox-fetchmail: qmailPrefix: %s' % qmailPrefix)
						fetchmailrcline2=('%s qvirtual %s' % (fetchmailrcline2,qmailPrefix))
					fetchmailrcline2=('%s no dns' % fetchmailrcline2)

					localDomainName=''
					if result[1] and result[1].has_key('localDomainName') and result[1]['localDomainName']:
						if result[1]['localDomainName']:
							localDomainName=result[1]['localDomainName'][0]
						univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'ox-fetchmail: localDomainName: %s' % localDomainName)
					if result[1] and not result[1].has_key('localRecipient'):
						if not localDomainName:
							localDomainName=listener.baseConfig['mail/hosteddomains']
						fetchmailrcline3=('	localdomains %s:' % localDomainName)

					if result[1] and result[1].has_key('remoteUser') and result[1]['remoteUser']:
						remoteUser=result[1]['remoteUser'][0]
						univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'ox-fetchmail: remoteUser: %s' % remoteUser)
						fetchmailrcline4=('	user \'%s\' there' % remoteUser)

					if result[1] and result[1].has_key('remotePassword') and result[1]['remotePassword']:
						remotePassword=result[1]['remotePassword'][0]
						# univention.debug.debug(univention.debug.LISTENER, univention.debug.ERROR, 'ox-fetchmail: remotePassword: %s' % remotePassword)
						fetchmailrcline4=('%s with password \'%s\'' % (fetchmailrcline4,remotePassword))

					if result[1] and result[1].has_key('localRecipient') and result[1]['localRecipient']:
						localRecipient=result[1]['localRecipient'][0]
						univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'ox-fetchmail: localRecipient: %s' % localRecipient)
						uid_result_id = ldapCon.search(base=ldapbase, filter='uidNumber=%s' % localRecipient, attr=['mailPrimaryAddress'])
						localRecipient=uid_result_id[0][1]['mailPrimaryAddress'][0]
						fetchmailrcline4=('%s is \'%s\' here' % (fetchmailrcline4,localRecipient))
					else:
						fetchmailrcline4=('%s is * here' % fetchmailrcline4)

					if result[1] and result[1].get('leaveMessagesOnServer'):
						leaveMessagesOnServer=result[1]['leaveMessagesOnServer'][0]
						univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'ox-fetchmail: leaveMessagesOnServer: %s' % leaveMessagesOnServer)
						if leaveMessagesOnServer == 'OK':
							fetchmailrcline4 += ' keep'
					if result[1] and result[1].get('useSSL'):
						useSSL=result[1]['useSSL'][0]
						univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'ox-fetchmail: useSSL: %s' % useSSL)
						if useSSL == 'OK':
							fetchmailrcline4 += ' ssl'

					univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'ox-fetchmail: oldcn: %s, thiscn: %s' % (deletedcn, thiscn))
					# if not thiscn[0] == deletedcn[0]:
					if True:
						# univention.debug.debug(univention.debug.LISTENER, univention.debug.ERROR, 'ox-fetchmail: line1: %s' % fetchmailrcline1)
						f.write('%s\n' % fetchmailrcline1)
						# univention.debug.debug(univention.debug.LISTENER, univention.debug.ERROR, 'ox-fetchmail: line2: %s' % fetchmailrcline2)
						f.write('%s\n' % fetchmailrcline2)
						if localDomainName:
							# univention.debug.debug(univention.debug.LISTENER, univention.debug.ERROR, 'ox-fetchmail: line3: %s' % fetchmailrcline3)
							f.write('%s\n' % fetchmailrcline3)
						# univention.debug.debug(univention.debug.LISTENER, univention.debug.ERROR, 'ox-fetchmail: line4: %s' % fetchmailrcline4)
						f.write('%s\n\n' % fetchmailrcline4)
				except:
					univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'ox-fetchmail: ID %s done.' % cn)

		f.close()
		os.chmod(fetchmailrc , 384)
		os.system('chown %s:%s %s' % ('fetchmail', 'root', fetchmailrc))
	finally:
		listener.unsetuid()


def postrun():
	listener.setuid(0)
	try:
		univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'ox-fetchmail: restarting fetchmail ...')
		os.spawnv(os.P_WAIT, '/etc/init.d/fetchmail', ['fetchmail', 'force-reload'])
	finally:
		listener.unsetuid()
